// This may look like C code, but it is really -*- C++ -*-
/*
Copyright (C) 1994 Free Software Foundation

   genclass program ported by Chris Anderson
   (original by Doug Lea (dl@rocky.oswego.edu))

Generate classes from prototypes

 genclass -[duv] -l [proto ...]
 genclass -[duv] -c [proto ...]
 genclass -[duv] -[1o:p:] type {ref|val} proto
 genclass -[duv] -[2o:p:] type1 {ref|val} type2 {ref, val} proto

This file is part of the GNU C++ Library.  This library is free
software; you can redistribute it and/or modify it under the terms of
the GNU Library General Public License as published by the Free
Software Foundation; either version 2 of the License, or (at your
option) any later version.  This library is distributed in the hope
that it will be useful, but WITHOUT ANY WARRANTY; without even the
implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU Library General Public License for more details.
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <GetOpt.h>
#include <dirent.h>
#include <String.h>
#include <SLList.h>
#include <bool.h>
#include "genclass.h"

int debug = FALSE;

void debug_print(char *msg)
{
    if (debug) { fprintf(stderr, "%s\n", msg); }
}
void errmsg(char *msg)
{
  fprintf(stderr, "ERROR: %s\n", msg);
}


void close_file(FILE *fp, char *f)
{
   int my_errno;

   if (fp != NULL)
   {
      my_errno = fclose(fp);
      if (my_errno != 0)
      {
         fprintf(stderr,"ERROR: closing file: %s, error: %i\n", f , my_errno);
         perror(f);
      }
   }
}

FILE *open_file(char *f, char *m)
{
  // open a file -- Return pointer to FILE upon success

   FILE *fp = NULL;

   if (f != NULL)
   {
      if ((fp = fopen(f,m)) == NULL)
        perror(f);
  }
   return fp;
}


int accessOK(String access)
{
  return (access == "ref" || access == "val");
}

/* Will return first prototype file it can find in the dir paths specified
   be dp
   */
String findProto(String dp[], String protoname)
{
    int i=0;
    FILE *fp;
    int gotHeader, gotSource;
    /* Try to find the first .ccP and .hP prototype combo and return filename
       without extensions
       */
    while( dp[i] != "") {
        gotHeader=FALSE;
        gotSource=FALSE;
        
        /* Check the .hP file */
        if ((fp = open_file(dp[i]+"/"+protoname+".hP", "r")) != NULL) {
            close_file(fp, dp[i]+"/"+protoname);
            gotHeader=TRUE;
        }
        /* Check the .ccP file */        
        if ((fp = open_file(dp[i]+"/"+protoname+".ccP", "r")) != NULL) {
            close_file(fp, dp[i]+"/"+protoname);
            gotSource=TRUE;
        }
        if (gotHeader && gotSource)
            return(dp[i]+"/"+protoname);
        else if (gotHeader) {
            cerr << "Found .hP file without .ccP file" << endl;
            return(dp[i]+"/"+protoname);
        }
        i++;
    }
    return "";
}

// This checks a file to see if this prototype requires two values or one
int checkclass(FILE *IN)
{
    char line[LINESIZE];
    
    while(fgets(line,LINESIZE-1, IN) != NULL)
    {
        String buf=line;
        if (buf.contains("<C&>") || buf.contains("<C>"))
            return(2);
    }
    return (1);
}


int create1class(FILE *IN, String type, String typeacc, String prefix, FILE *OUT)
{
    char line[LINESIZE];
    int err=0;
    if (checkclass(IN) == 2) {
        String msg = "class requires the -2 syntax [ requires two arguments ]";
        errmsg(msg);
        err=1;
    } else {
        rewind(IN);
        while(fgets(line,LINESIZE-1,IN) != NULL) {
            String buf=line;

            buf.gsub("<T>",type);
            buf.gsub("<T&>",typeacc);
            buf.gsub(type, prefix);
            fprintf(OUT, "%s", (char *) buf);
        }
        fclose(IN);
        fclose(OUT);
    }
    return(err);
}


int create2class(FILE *IN, String prefix, String type1, String typeacc1,
		  String type2, String typeacc2, FILE *OUT)
{
    char line[LINESIZE];
    int err=0;

    if (checkclass(IN) == 1) {
        String msg = "class requires the -1 syntax [ requires one argument ]";
        errmsg(msg);
        err=1;
    } else {
        rewind(IN);
        while(fgets(line,LINESIZE-1,IN) != NULL)  {
            String buf=line;

            buf.gsub("<T>",type1);
            buf.gsub("<T&>",typeacc1);
            buf.gsub("<C>",type2);
            buf.gsub("<C&>",typeacc2);
            buf.gsub(type1+"."+type2+".",prefix);
            fprintf(OUT, "%s", (char *) buf);
        }
        fclose(IN);
        fclose(OUT);
    }
    return(err);
}


int gen1class(String dp[], String oname, String prefix, String args[])
{
  int error = 0;
  FILE *IN_ccfile,  *IN_hfile;
  FILE *OUT_ccfile, *OUT_hfile;
  
  debug_print("Generate one parameter class prototype");
  
  String type=args[0], access=args[1], proto=args[2];

  if (!accessOK(access)) {
      String msg = "must specify type access as \"ref\" or \"val\"";
      errmsg(msg);
      error++;
  }
  
  if (prefix == "") { prefix=type; }
  else { prefix+=type; }
  
  String pname = findProto(dp,proto);
  if (pname == "") {
      String msg = "no prototype for class " + proto;
      errmsg(msg);
      error++;
  }
  else {
      String msg = "Found: " + pname;
      cerr << msg << endl;
  }
  
  String typeacc;
  if (access == "ref") { typeacc = type+"&"; }
  else { typeacc = type; }  

  if ( ((IN_ccfile = open_file(pname+".ccP","r")) != NULL) &&
       ((OUT_ccfile = open_file(oname + c_ext,"w")) != NULL) )
  {
      int rc=create1class(IN_ccfile, type, typeacc, prefix, OUT_ccfile);

      if (rc != 0) {
          error++;
          errmsg("no .cc file created\n");          
          remove(oname+c_ext);
      }
  }
  else {
      error++;
      fprintf(stderr, "no .cc file created\n");
      remove(oname+c_ext);
  }
  
  if ( ((IN_hfile = open_file(pname+".hP","r")) != NULL) &&
       ((OUT_hfile = open_file(oname + h_ext,"w")) != NULL) )
  {
     int rc = create1class(IN_hfile, type, typeacc, prefix, OUT_hfile);
     
     if (rc != 0) {
         error++;
         errmsg("no .h file created\n");
         remove(oname+h_ext);
     }
  }
  else {
      error++;
      fprintf(stderr,"no .h file created");
      remove(oname+h_ext);
  }  

  if (error == 0) {
      cerr << "Edit the .cc file to include " <<  oname+h_ext << endl;
  }
        
  return error;
}

int gen2class(String dp[], String oname, String prefix, String args[])  {
    
  int error = 0;
  FILE *IN_ccfile, *IN_hfile;
  FILE *OUT_ccfile, *OUT_hfile;
  
  debug_print("Generate two parameters class prototype");
  
  String type1=args[0], access1=args[1];
  String type2=args[2], access2=args[3];
  String proto=args[4];

  if (prefix == "") { prefix=type1+type2; }
  else {  prefix+=proto; }
  
  if (!accessOK(access1)) {
      String msg = "must specify type1 access as \"ref\" or \"val\"";
      errmsg(msg);
      error++;
  }
	  
  if (!accessOK(access2)) {
      String msg = "specify type2 access as \"ref\" or \"val\"";
      errmsg(msg);
      error++;
  }
	  
  String pname = findProto(dp,proto);
  if (pname == "") {
      String msg = "no prototype for class " + proto;
      errmsg(msg);
      error++;
  }
  else {
      String msg = "Found: " + pname;
      cerr << msg << endl;
  }
  
  String typeacc1, typeacc2;
  if (access1 == "ref") {typeacc1 = type1+"&";}
  else { typeacc1 = type1; }
  if (access2 == "ref") {typeacc2 = type2+"&";}
  else { typeacc2 = type2; }  

  if ( ((IN_ccfile = open_file(pname+".ccP","r")) != NULL) &&
       ((OUT_ccfile = open_file(oname + c_ext,"w")) != NULL) )
  {
      int rc=create2class(IN_ccfile, prefix, type1, typeacc1,
                                              type2, typeacc2, OUT_ccfile);
      if (rc != 0) {
          error++;
          errmsg("no .cc file created\n");          
          remove(oname+c_ext);
      }
  }
  else {
      error++;
      fprintf(stderr,"no .cc file created\n");
      remove(oname+c_ext);
  }
  
  if ( ((IN_hfile = open_file(pname+".hP","r")) != NULL) &&
       ((OUT_hfile = open_file(oname + h_ext,"w")) != NULL) )
  {
      int rc=create2class(IN_hfile, prefix, type1, typeacc1,
                                             type2, typeacc2, OUT_hfile);
     
     if (rc != 0) {
         error++;
         errmsg("no .h file created\n");
         remove(oname+h_ext);
     }
  }
  else {
      error++;
      fprintf(stderr, "no .h file created\n");
      remove(oname+h_ext);
  }
  
  if (error == 0) {
      cerr << "Edit the .cc file to include " <<  oname+h_ext << endl;
  }
  
  return error;
}

SLList<String> sort_proto_list(SLList<String> pl)
{
  SLList<String> nl;

  for (Pix i=pl.first(); i!=0; pl.next(i))
  {
      if (nl.empty()) nl.prepend(pl(i));
      else
      {
	  for (Pix prev=0, curr=nl.first(); curr!=0; prev=curr, nl.next(curr))
          {
	      if (pl(i) == nl(curr)) { break; }
	      else if (nl(curr) > pl(i))
              { 
		  nl.ins_after(prev,pl(i)); 
		  break;
              }
          }
	  if (curr == 0) { nl.append(pl(i)); }
      }
  }
  return nl;
}
	    
      
SLList<String> get_proto_list(String dp[], String proto[])
{
    DIR *d;
    int i=0;
    SLList<String> pl;
    struct dirent *de;

    while (proto[i] != "")   {
        Regex p = (proto[i] == "all" ? (".+") : (proto[i]));

        int k=0;
        while( dp[k] != "") {
            if ((d = opendir(dp[k])) != NULL) {
                while (de = readdir(d))  {
                    String name = de->d_name;
                    if (name.contains(ccP_FILE) || name.contains(hP_FILE)) {
                        name.del(ccP_FILE);
                        name.del(hP_FILE);
                        if (name.contains(p))
                            pl.append(name);
                    }
                }
                closedir(d);
            }
            k++;
	}
        i++;
    }
    pl = sort_proto_list(pl);
    return pl;
}

void print_class(String dp[], String select[], ACTIONS type)
{
    SLList<String> plist;

    switch(type)
    {
        case LIST: 
        {
            plist = get_proto_list(dp, select);
            for (Pix k=plist.first(); k!=0; plist.next(k))
                cout << plist(k) << endl;
            break;
        }
    
        case CAT:
        {
            int i;
            int col = 0;

            cout << "Catalog of genclass class templates" << endl;
            cout << "directories searched:" << endl;
            for (i=0; dp[i] != ""; ++i)
                cout << "   " << dp[i] << endl;
            cout << "selecting: ";
            for (i=0; select[i] != ""; ++i)
                cout << select[i] << " ";
            cout << endl;
            cout << "classes available: " << endl;

            plist = get_proto_list(dp, select);

            for (Pix k=plist.first(); k!=0; plist.next(k))
            {
                if (col == 0) { cout << "    "; }
                cout << plist(k) << " ";
                col++;
                if (col == 8) { col = 0; cout << endl; }
            }
            if (col != 0) { cout << endl; }
            break;
        }
    }
}


/* MAIN PROGRAM FOR genclass */
main(int argc, char **argv)
{
    ACTIONS action;
    int i, k, stop=FALSE;
    String OUT_filename, OUT_prefix;

// Invokes ctor `GetOpt (int argc, char **argv,  char *optstring);
// f - OUTPUT filename, u - Usage, v - version, d - debug on
    GetOpt getopt (argc, argv, "Uu\?VvDdLlCc12O:o:P:p:");
    int option_char;
    OUT_filename="";
    OUT_prefix="";
    action=ERR;
  
// Invokes member function `int operator ()(void);'
    while ((option_char = getopt ()) != EOF) {
        switch (toupper(option_char))
        {
            case 'D':  debug = TRUE; break;
              
        // Name of output file (without extension -- they are added)
            case 'O':  OUT_filename = getopt.optarg; break;
        // Prefix string to append to types
            case 'P':  OUT_prefix = getopt.optarg; break;              

        // Action to perform
            case 'L':  action=LIST;                  break;
            case 'C':  action=CAT;                   break;
            case '1':  action=ONE;                   break;
            case '2':  action=TWO;                   break;

        // Print version information
            case 'V':  fprintf(stderr, "%s\n", VERSION);  stop=TRUE; break;
                
        // Print usage information              
            case 'U':
            case '\?':  fprintf(stderr, "%s\n", USAGE);    stop=TRUE; break;                              
        // Print usage and version by default
            default:
                fprintf(stderr, "%s\n", VERSION);
                fprintf(stderr, "%s\n", USAGE);
                exit(1);
        }
    }

// If user requested VERSION or USAGE just stop so they can marvel and ponder!!
    if (stop) exit(0);

    if (action == ERR) {
        errmsg("Incorrect usage: Must specify -l -c -1 -2");
        fprintf(stderr, "%s\n", VERSION);
        fprintf(stderr, "%s\n", USAGE);
        exit(1);
    }

// User must specify an filename (WITHOUT Extension) to put the
// the results into
    if ((action == ONE || action == TWO) && OUT_filename == "")   {
        errmsg("You must specify an output filename (without extension)");
        fprintf(stderr, "%s\n", USAGE);
        exit(1);
    }

// Find DJGPP.ENV and get C++ directories if ENV set
// Need to have a section called genclass
  
    char *gendir = getenv("PROTODIR");

// Get directories to search for *.hP and *.ccP
// Need code to search DJGPP.ENV for C++ directories
    String dirpath[4];
    dirpath[0] = "/usr/local/lib/g++-include/gen";
    dirpath[1] = gendir;
    dirpath[2] = ".";


// Put the rest of the arguments into an array
    int numargs = getopt.nargc - getopt.optind;
    String arglist[numargs+2];
    k=0;
    while(getopt.optind < getopt.nargc) 
        arglist[k++] = getopt.nargv[getopt.optind++];

    numargs = k;

    if (debug) {
        cout << "directories searched:" << endl;
        for (int i=0; dirpath[i] != ""; ++i)
            cout << "   " << dirpath[i] << endl;
    }

    int error = 0;
    switch(action)
    {
        case LIST:
        case CAT:
            if (numargs == 0) { arglist[0] = "all"; }
            print_class(dirpath,arglist,action);
            break;
      
        case ONE: // genclass type {ref|val} proto out_prefix
            if (numargs != 3) {
                errmsg("incorrect number of arguments for -1 type");
                error++;
            }
          
            if (error == 0) {
                if (gen1class(dirpath, OUT_filename, OUT_prefix, arglist) != 0)
                    error++;
            }
            break;
      
        case TWO: // genclass type1 {ref|val} type2 {ref|val} proto out_prefix
        {
            if (numargs != 5) {
                String msg = "incorrect number of arguments";
                errmsg(msg);
                error++;
            }
	  
            if (error == 0) {
                if (gen2class(dirpath, OUT_filename, OUT_prefix, arglist) != 0)
                    error++;
            }
            break;
        }
    } // End Switch
    
    if (error) {
        fprintf(stderr, "%s\n", USAGE);
    }
    
    exit(error);    
}
