/* Project SWORD
   V2.0

   SubSystem : Basic objects, General use mecanisms
   File      : LibSrc/Mecanism/Disk.CC
   Author    : Eric NICOLAS
   Overview  : Object for accessing disks or other DOS peripherals
   UpDate    : Oct 02, 1995

** Copyright (C) 1993,1995 The SWORD Group
**
** This file is distributed under the terms listed in the document
** "copying.en". A copy of "copying.en" should accompany this file.
** if not, a copy should be available from where this file was obtained.
** This file may not be distributed without a verbatim copy of "copying.en".
**
** This file is distributed WITHOUT ANY WARRANTY; without even the implied
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#include "Common/Common.H"
#include "Mecanism/Disk.H"

void TDisk::Open(char* Name, int Acces)
{
#ifdef __TURBOC__
  ToAntiSlash(Name);
#endif
  char Access[10];
  switch(Acces & (stCreate | stOpen))
  { case stCreate :
      strcpy(Access,"w");
      break;
    case stOpen :
      strcpy(Access,"r");
      break;
    default :
      Error=erDiskAccess_Open;
      return;
  }
  strcat(Access,"b");
  if ((FileHandle=fopen(Name,Access))==NULL) Error=erDiskAccess_Open;
#ifdef __TURBOC__
  ToSlash(Name);
#endif
}

void TDisk::Close(void)
{ if (!Error)
    if (!(fclose(FileHandle))) Error=erDiskAccess_Close;
}

void TDisk::Write(void* Buffer, int Size)
{ if (!Error)
    if (fwrite(Buffer,1,Size,FileHandle)!=Size) Error=erDiskAccess_Write;
}

void TDisk::Read(void* Buffer, int Size)
{ if (!Error)
  { LastRead=fread(Buffer,1,Size,FileHandle);
    if (LastRead!=Size)
      if (LastRead) Error=erDiskAccess_EOF;
	           else Error=erDiskAccess_Read;
  }
}

// ------------------------------------------------------------------------

long TDisk::Seek(long Depl, char Mode)
{ if (!Error)
    if (fseek(FileHandle,Depl,Mode))
    { Error=erDiskAccess_Seek;
      return 0L;
    }
    else return Depl;
  return 0L;
}

long TDisk::Position(void)
{ return ftell(FileHandle);
}

char CrLf[2] = { 13, 10 };
char Tmp[1024];

void TDisk::ReadLn(char *Buffer, int MaxLength)
{ if (!Error) return;
  int Length=min(MaxLength+1,1024);
  // Lit la future chane dans un buffer
  int p_begin=Position();
  memset(Tmp,0,Length);
  Read(Tmp,Length);
  // Cherche la fin de la ligne
  int p=0;
  while((Tmp[p]!=13)&&(Tmp[p]!=10)&&(p<Length)) p++;
  if (p==Length)
  { // Aucun retour chariot dans les Length premiers caractres
    if (Error)
    { // Il y a eu erreur : on a atteint la fin du fichier
      // Donc le contenu de Tmp, avec des 0 aprs est OK.
      strcpy(Buffer,Tmp);
    }
    else
    { // Il n'y a pas eu d'erreur de lecture, c'est donc que la ligne 
      // lire tait plus longue que le buffer rserv
      Error=erDiskAccess_LineTooLong;
    }
  }
  else
  { Error=0;
    // On a trouv la fin de la ligne
    strncpy(Buffer,Tmp,p);
    Buffer[p]=0;
    // Passe le couple Cr,Lf
    if (Tmp[p]==13)
      if (Tmp[p+1]==10) p++;
    if (Tmp[p]==10)
      if (Tmp[p+1]==13) p++;
    // Avance le pointeur de fichier  la bonne position
    Seek(p+p_begin+1,stBegining);
  }
}

void TDisk::WriteLn(char *Buffer)
{ Write(Buffer,strlen(Buffer));
  Write(CrLf,2);
}
