		.file "mouse.s"
		.text
m_start_of_text:
mouse_isr:
		# save interrupt state
		pushfl
		pushal
		pushl	%ds
		pushl	%es
		pushl	%fs
		pushl	%gs

		# set up real mode return address
		movl	(%esi),%eax
		movl	%eax,%es:42(%edi)
		addw	$4,%es:46(%edi)

		# push mouse parameters to stack
		pushl	%es:0x10(%edi)  # bx = buttons
		pushl	%es:0x00(%edi)  # di = vertical mickey count
		pushl	%es:0x04(%edi)  # si = horizontal mickey count
		pushl	%es:0x14(%edi)  # dx = vertical cursor position
		pushl	%es:0x18(%edi)  # cx = horizontal cursor position
		pushl	%es:0x1c(%edi)	# ax = condition mask causing call

		# setup %ds and %es to point to our data segment
		.byte	0x2e			# %cs prefix, for gas botch
		movw	m_data_segment,%ds
		movw	m_data_segment,%es
		# call mouse handler if valid
		movl	mouseCallback,%eax
		orl		%eax,%eax
		jz		1f
		call	%eax
1:		add		$24,%esp

		# restore interrupt state
		popl	%gs
		popl	%fs
		popl	%es
		popl	%ds
		popal
		nop
		popfl
		iret
m_end_of_text:
_m_init:.globl _m_init
		pushl	%ebx
		pushl	%ecx
		movw	$0x0000,%ax
		int		$0x33
		orw		%ax,%ax
		jz		err
		movw	%bx,numButtons
		movw	%ds,%ax
		movw	%ax,m_data_segment
		# get ds base
		movzwl	%ax,%ebx		# selector
		movw	$0x0006,%ax
		int		$0x31
		jc		err
		# cx:dx base
		movw	%cx,%bx
		shll	$16,%ebx
		movw	%dx,%bx
		movl	%ebx,%edx		# base
		# lock text
		# bx:cx address
		# si:di size
		leal	m_start_of_text(%edx),%eax
		movw	%ax,%cx
		shrl	$16,%eax
		movw	%ax,%bx
		movl	$m_end_of_text,%eax
		subl	$m_start_of_text,%eax
		movw	%ax,%di
		shrl	$16,%eax
		movw	%ax,%si
		movw	$0x600,%eax
		int		$0x31
		jc		err
		# lock data
		leal	m_start_of_data(%edx),%eax
		movw	%ax,%cx
		shrl	$16,%eax
		movw	%ax,%bx
		movl	$m_end_of_data,%eax
		subl	$m_start_of_data,%eax
		movw	%ax,%di
		shrl	$16,%eax
		movw	%ax,%si
		movw	$0x600,%eax
		int		$0x31
		jc		err
		jmp		1f
err:	movl	$-1,%eax
		jmp		m_exit
		# allocate callback
1:		pushl	%ds
		pushl	%cs
		popl	%ds
		movl	$mouse_isr,%esi
		movl	$cb_regs,%edi
		movw	$0x0303,%ax
		int		$0x31
		popl	%ds
		jc		err
		movw	%cx,cb_segment
		movw	%dx,cb_offset
		xorl	%eax,%eax
m_exit:
		popl	%ecx
		popl	%ebx
		ret

_m_uninit:.globl _m_uninit
		pushl	%ebx
		pushl	%ecx
		# reset mouse
		movw	$0,%ax
		int		$0x33
		# free callback
		movw	cb_segment,%cx
		movw	cb_offset,%dx
		movw	$0x0304,%ax
		int		$0x31
		movw	$0,cb_segment
		movw	$0,cb_offset
		# unlock memory
		# get ds base
		movw	%ds,%bx
		movzwl	%bx,%ebx		# selector
		movw	$0x0006,%ax
		int		$0x31
		jnc		1f
err2:	movl	$-1,%eax
		jmp		m_exit
		# cx:dx base
1:		movw	%cx,%bx
		shll	$16,%ebx
		movw	%dx,%bx
		movl	%ebx,%edx		# base

		# unlock text
		# bx:cx address
		# si:di size
		leal	m_start_of_text(%edx),%eax
		movw	%ax,%cx
		shrl	$16,%eax
		movw	%ax,%bx
		movl	$m_end_of_text,%eax
		subl	$m_start_of_text,%eax
		movw	%ax,%di
		shrl	$16,%eax
		movw	%ax,%si
		movw	$0x601,%eax
		int		$0x31
		jc		err2

		# unlock data
		leal	m_start_of_data(%edx),%eax
		movw	%ax,%cx
		shrl	$16,%eax
		movw	%ax,%bx
		movl	$m_end_of_data,%eax
		subl	$m_start_of_data,%eax
		movw	%ax,%di
		shrl	$16,%eax
		movw	%ax,%si
		movw	$0x601,%eax
		int		$0x31
		jc		err2
		xorl	%eax,%eax
		jmp		m_exit

_m_getButtons:.globl _m_getButtons
		movzwl	numButtons,%eax
		ret

_m_showCursor:.globl _m_showCursor
		testw	$0xffff,numButtons
		jnz		1f
		ret
1:
		movw	$0x0001,%ax
		int		$0x33
		ret

_m_hideCursor:.globl _m_hideCursor
		testw	$0xffff,numButtons
		jnz		1f
		ret
1:
		movw	$0x0002,%ax
		int		$0x33
		ret

_m_getPos:.globl _m_getPos
		testw	$0xffff,numButtons
		jnz		1f
		xorl	%eax,%eax
		ret
1:
		pushl	%ebp
		movl	%esp,%ebp
		pushl	%esi
		pushl	%edi
		pushl	%ebx
		pushl	%ecx
		movw	$0x0003,%ax
		int		$0x33
		movl	8(%ebp),%esi
		movl	12(%ebp),%edi
		movw	%cx,(%esi)
		movw	%dx,(%edi)
		movzwl	%bx,%eax
		popl	%ecx
		popl	%ebx
		popl	%edi
		popl	%esi
		leave
		ret

_m_setPos:.globl _m_setPos
		testw	$0xffff,numButtons
		jnz		1f
		ret
1:
		pushl	%ebp
		movl	%esp,%ebp
		pushl	%ecx
		movw	$0x0004,%ax
		movw	8(%ebp),%cx
		movw	12(%ebp),%dx
		int		$0x33
		popl	%ecx
		leave
		ret

_m_getPress:.globl _m_getPress
		testw	$0xffff,numButtons
		jnz		1f
		ret
1:
		pushl	%ebp
		movl	%esp,%ebp
		pushl	%esi
		pushl	%edi
		pushl	%ebx
		pushl	%ecx
		movw	$0x0005,%ax
		int		$0x33
		movl	8(%ebp),%esi
		movl	12(%ebp),%edi
		movw	%bx,(%esi)
		movw	%cx,(%edi)
		movl	16(%ebp),%esi
		movl	20(%ebp),%edi
		movw	%dx,(%esi)
		movw	%ax,(%edi)
		popl	%ecx
		popl	%ebx
		popl	%edi
		popl	%esi
		leave
		ret

_m_getRelease:.globl _m_getRelease
		testw	$0xffff,numButtons
		jnz		1f
		ret
1:
		pushl	%ebp
		movl	%esp,%ebp
		pushl	%esi
		pushl	%edi
		pushl	%ebx
		pushl	%ecx
		movw	$0x0006,%ax
		int		$0x33
		movl	8(%ebp),%esi
		movl	12(%ebp),%edi
		movw	%bx,(%esi)
		movw	%cx,(%edi)
		movl	16(%ebp),%esi
		movl	20(%ebp),%edi
		movw	%dx,(%esi)
		movw	%ax,(%edi)
		popl	%ecx
		popl	%ebx
		popl	%edi
		popl	%esi
		leave
		ret

_m_setHRange:.globl _m_setHRange
		testw	$0xffff,numButtons
		jnz		1f
		ret
1:
		pushl	%ebp
		movl	%esp,%ebp
		pushl	%ecx
		movw	$0x0007,%ax
		movw	8(%ebp),%cx
		movw	12(%ebp),%dx
		int		$0x33
		popl	%ecx
		leave
		ret

_m_setVRange:.globl _m_setHRange
		testw	$0xffff,numButtons
		jnz		1f
		ret
1:
		pushl	%ebp
		movl	%esp,%ebp
		pushl	%ecx
		movw	$0x0008,%ax
		movw	8(%ebp),%cx
		movw	12(%ebp),%dx
		int		$0x33
		popl	%ecx
		leave
		ret

_m_setUserFunction:.globl _m_setUserFunction
		testw	$0xffff,numButtons
		jnz		1f
		ret
1:
		pushl	%ebp
		movl	%esp,%ebp
		pushl	%edi
		pushl	%ebx
		pushl	%ecx
		movl	8(%ebp),%eax
		movl	%eax,mouseCallback
		movl	$0,cb_regs+0x2e
		movw	$0x000c,cb_regs+0x1c
		movw	cb_offset,%ax
		movw	%ax,cb_regs+0x14
		movw	cb_segment,%ax
		movw	%ax,cb_regs+0x22
		movw	12(%ebp),%ax
		movw	%ax,cb_regs+0x18
		movw	$0x0300,%ax
		movw	$0x0033,%bx
		movw	$0,%cx
		movl	$cb_regs,%edi
		int		$0x31
		popl	%ecx
		popl	%ebx
		popl	%edi
		leave
		ret

		.data
m_start_of_data:
mouseCallback:
		.long	0
cb_offset:
		.short	0
cb_segment:
		.short	0
cb_regs:
		.fill	50,1,0
m_data_segment:
		.short	0
numButtons:
		.short	0
m_end_of_data:
