/*****************************************************************************\
 * Interface to 32-bit executable (from stub.asm)
 *
 *   cs:eip	according to COFF header
 *   ds		32-bit data segment for COFF program
 *   fs		selector for our data segment (fs:0 is stubinfo)
 *   ss:sp	our stack (ss to be freed)
 *   <others>	All unspecified registers have unspecified values in them.
\*****************************************************************************/

	.file "crt0.s"

#include "stubinfo.h"

	.comm	__stklen, 4
	.comm	__stubinfo, 4
	.lcomm	sel_buf, 8
	/* ___djgpp_ds_alias defined in go32/exceptn.s */

	.data
sbrk16_first_byte:
#include "sbrk16.ah"
sbrk16_last_byte:

sbrk16_api_ofs:
	.long	0
sbrk16_api_seg:
	.word	0

#define EXIT16 1
#if EXIT16	
exit16_first_byte:
#include "exit16.ah"
exit16_last_byte:
#endif

.globl   __jhw_using_fat_ds   /*JHW main() sets this to true when we are using the fat ds, so ds limit is not alterred*/
__jhw_using_fat_ds:   /*JHW main() sets this to true when we are using the fat ds, so ds limit is not alterred*/
        .long 0         /*False=not using fat ds yet*/
        .align 2
.globl   __jhw_ds_req_limit   /*JHW last requested limit for ds, so we can restore the correct limit after using the fat ds*/
__jhw_ds_req_limit:   /*JHW last requested limit for ds, so we can restore the correct limit after using the fat ds*/
        .long 0   
        .align 2


hook_387_emulator:
	.long	___emu387_load_hook

	.text

	.globl	start
start:
	pushl	%ds			/* set %es same as %ds */
	popl	%es			/* push/pop 4 bytes shorter than ax */

	movl	$edata, %edi		/* set all BSS bytes to zero */
	movl	$end, %ecx
	subl	%edi, %ecx
	xorl	%eax, %eax		/* Zero fill value */
	shrl	$2, %ecx		/* div 4 Longwords not bytes */
	cld
	rep
	stosl


/* Create an alias for DS to be used by real-mode callbacks (exception handler messes with DS itself) */

	movw	%ds, %bx
	movw	$0x000a, %ax
	int	$0x31
	jnc	ds_alias_ok
	movb	$0x4c, %ah
	int	$0x21
ds_alias_ok:
	movw	%ax, ___djgpp_ds_alias
	movl	%eax, %ebx
	movw	$0x0009, %al
	movw	%cs, %cx	/* get CPL from %cs */
	andw	$3, %cx
	shlw	$5, %cx		/* move it into place */
	orw	$0xc093, %cx
	int	$0x31		/* set access rights for alias */

/* Allocate some DOS memory and copy our sbrk helper into it. */

	movzwl	sbrk16_first_byte+8, %ebx
	shrl	$4, %ebx
	movw	$0x0100, %ax
	int	$0x31
	jnc	dos_alloc_ok
	movb	$0x4c, %ah
	int	$0x21
dos_alloc_ok:
	movw	%cs, sbrk16_first_byte+2	/* store API information */
	movw	%ds, sbrk16_first_byte+4
	movw	%dx, sbrk16_first_byte+6	/* selector for allocated block */

	pushl	%es				/* move the data */
	movw	%dx, %es
	movl	$(sbrk16_last_byte - sbrk16_first_byte), %ecx
	shrl	$2,%ecx
	movl	$sbrk16_first_byte, %esi
	xorl	%edi, %edi
	cld
	rep
	movsl
	popl	%es

	movzwl	sbrk16_first_byte, %eax		/* calculate API address */
	movl	%eax, sbrk16_api_ofs

	movw	%dx, %bx			/* dos memory selector */
	movw	$0x000b, %ax			/* get descriptor */
	movl	$sel_buf, %edi
	int	$0x31

	andb	$0xbf, sel_buf+6		/* make 16-bit */
	andb	$0xf0, sel_buf+5		/* remove old type */
	orb	$0x0a, sel_buf+5		/* set new type to code/read */

	xorl	%eax, %eax			/* allocate new selector */
	movw	$0x0001, %cx
	int	$0x31
	movw	%ax, sbrk16_api_seg

	movw	%ax, %bx
	movw	$0x000c, %ax			/* set descriptor */
	movl	$sel_buf, %edi
	int	$0x31

/* Initialize the brk/sbrk variables */

	movl	$end, __what_size_app_thinks_it_is
	.byte 0x64 /* fs: */
	movl	STUBINFO_INITIAL_SIZE, %eax
	movl	%eax, __what_size_dpmi_thinks_we_are

	.byte 0x64 /* fs: */
	movl	STUBINFO_MEMORY_HANDLE, %eax
	movl	%eax, __dpmi_memory_handle

	.byte 0x64 /* fs: */			/* copy stubinfo into local memory */
	movl	STUBINFO_SIZE, %eax
	pushl	%eax
	call	___sbrk
	movl	%eax, __stubinfo
	movl	%eax, %edi
	.byte 0x64 /* fs: */
	movl	STUBINFO_SIZE, %ecx
	shrl	$2, %ecx
	xorl	%esi, %esi			/* Zero */
	pushl	%ds
	pushl	%fs
	popl	%ds
	cld
	rep
	movsl
	popl	%ds

#if 0						/* done in crt1.c */
	.byte 0x64 /* fs: */			/* set up _go32_info_block structure */
	movzwl	STUBINFO_MINKEEP, %eax
	movl	%eax, __go32_info_block+16	/* .size_of_transfer_buffer */
	.byte 0x64 /* fs: */
	movzwl	STUBINFO_DS_SEGMENT, %eax
	shll	$4, %eax
	movl	%eax, __go32_info_block+12	/* .linear_address_of_transfer_buffer */
	xorl	%eax, %eax
	movl	$1, %ecx
	int	$0x31
	jc	no_selector
	movw	%ax, __go32_info_block+26	/* .selector_for_linear_memory */
	movl	%eax, %ebx
	movl	$8, %eax
	movl	$0x0f, %ecx
	movw	$0xffff, %dx	
	int	$0x31				/* Set limit 1Mb */
no_selector:
#endif

	movl	__stklen, %eax		/* get program-requested stack size */
	.byte 0x64 /* fs: */
	movl	STUBINFO_MINSTACK, %ecx	/* get stub-requested stack size */
	cmpl	%ecx, %eax
	jge	use_stubinfo_stack_size	/* use the larger of the two */
	movl	%ecx, %eax
	movl	%eax, __stklen		/* store the actual stack length */
use_stubinfo_stack_size:
	pushl	%eax
	call	___sbrk			/* allocate the memory */
	cmpl	$-1, %eax
	je	no_memory
	addl	__stklen, %eax
	movw	%ds, %dx		/* set stack */
	movw	%dx, %ss

	movl	%eax, %esp

	xorl	%ebp, %ebp
	call	___crt1_startup		/* run program */
	jmp	exit

no_memory:
	movb	$0xff, %al
	jmp	exit

/*-----------------------------------------------------------------------------*/

#define FREESEL(x) movw x, %bx; movw $0x0001, %ax; int $0x31

	.global	__exit
__exit:
	movb	4(%esp), %al
exit:
	movb	%al, %cl
	FREESEL(sbrk16_api_seg)		/* sbrk cs */
	FREESEL(__go32_info_block+26)	/* selector for linear memory */
	FREESEL(___djgpp_ds_alias)	/* DS alias for rmcb exceptions */
	movw	sbrk16_first_byte+6,%dx /* selector for allocated DOS mem */
	movw	$0x101, %ax
	int	$0x31			/* Free block and selector */
	xorl	%ebp, %ebp
	movl	__stubinfo, %edx
#ifdef EXIT16
	movw	STUBINFO_CS_SELECTOR(%edx), %ax
	movw	%ax, sbrk16_api_seg
	xorl	%edi, %edi
	movl	%edi, sbrk16_api_ofs	/* Offset is zero */

	movw	STUBINFO_DS_SELECTOR(%edx), %es
	movb	%cl, %dl
	movl	$exit16_first_byte, %esi
	movl	$(exit16_last_byte - exit16_first_byte), %ecx
	cld
	rep
	movsb

	movl	__dpmi_memory_handle, %edi
	movl	__dpmi_memory_handle+2, %esi

	movw	%es,%ax
	movw	%ax,%ss
	movl	$0x400,%esp		/* Transfer buffer >= 1024 bytes */
	FREESEL(%ds)
	movw	%cs, %bx
/* Call exit procedure with BX=32-bit CS; SI+DI=32-bit handle; DL=exit status */
	.byte 0x2e
	ljmp	sbrk16_api_ofs
#else
	FREESEL(STUBINFO_DS_SELECTOR(%edx))
	FREESEL(STUBINFO_CS_SELECTOR(%edx))
	movb	%cl, %al
	movb	$0x4c, %ah
	int	$0x21
#endif

/*-----------------------------------------------------------------------------*/

	.lcomm	__what_size_app_thinks_it_is, 4
	.lcomm	__what_we_return_to_app_as_old_size, 4
	.lcomm	__what_size_dpmi_thinks_we_are, 4
	.lcomm	__what_size_we_ask_dpmi_to_be, 4
	.lcomm	__dpmi_memory_handle, 4
	.lcomm	__base_address, 4
	.lcomm	__old_base_address, 4

#if 0
brk_hook_ret:
	ret
	.globl ___sbrk_brk_hook
___sbrk_brk_hook:
	.long	brk_hook_ret
#endif

	.global	___sbrk
	.align	2
___sbrk:
	movl	__what_size_app_thinks_it_is, %eax
	addl	4(%esp), %eax
	jmp	brk_common

	.globl	___brk
	.align	2
___brk:
	movl	4(%esp), %eax

brk_common:
	pushl	%esi
	pushl	%edi
	pushl	%ebx

	movl	%eax, %edx
	movw	$0x0900, %ax					/* disable interrupts */
	int	$0x31
	pushl	%eax
	movl	%edx, %eax

	movl	__base_address, %edx
	movl	%edx, __old_base_address

	movl	__what_size_app_thinks_it_is, %edx		/* save info */
	movl	%edx, __what_we_return_to_app_as_old_size
	movl	%eax, __what_size_app_thinks_it_is

	addl	$0x0000ffff, %eax				/* round up to 64K block */
	andl	$0xffff0000, %eax
	cmpl	__what_size_dpmi_thinks_we_are, %eax		/* don't bother shrinking */
	jle	brk_nochange
	movl	%eax, __what_size_we_ask_dpmi_to_be		/* save for later */

	movw	__dpmi_memory_handle, %di			/* request new size */
	movw	__dpmi_memory_handle+2, %si
	movw	__what_size_we_ask_dpmi_to_be, %cx
	movw	__what_size_we_ask_dpmi_to_be+2, %bx

	lcall	sbrk16_api_ofs
	jc	brk_error

	movw	%di, __dpmi_memory_handle			/* store new handle */
	movw	%si, __dpmi_memory_handle+2
	movw	%cx, __base_address				/* store new base address */
	movw	%bx, __base_address+2	
	decl	__what_size_we_ask_dpmi_to_be			/* limit now, not size */

	movw	$0x0008, %ax					/* reset CS limit */
	movw	%cs, %bx
								/*JHW since CS!=DS dont need a __jhw_cs_req_limit*/
	movl	__what_size_we_ask_dpmi_to_be, %edx		/*JHW get the whole size, and low word16 into %dx*/
	movl	%edx, __jhw_ds_req_limit			/*JHW Save the limit*/
	movw	__what_size_we_ask_dpmi_to_be+2, %cx
	int	$0x31

	testl	$1, __jhw_using_fat_ds				/*JHW are we using the fat ds?*/
	jnz	using_fat_ds					/*JHW using fat ds so skip setting limit*/

	movw	$0x0008, %ax					/* reset DS limit */
	movw	%ds, %bx
	int	$0x31
using_fat_ds:							/*JHW*/
	movw	$0x0008, %ax					/* reset DS alias limit */
	movw	___djgpp_ds_alias, %bx
	int	$0x31

	movw	$0x0007, %ax					/* reset DS alias base */
	movw	___djgpp_ds_alias, %bx
	movw	__base_address, %dx
	movw	__base_address+2, %cx
	int	$0x31
	

	testb	$0x40, __crt0_startup_flags	/* include/crt0.h */
	jz	no_fill_sbrk_memory
	pushl	%ds
	popl	%es

	movl	__what_size_dpmi_thinks_we_are, %edi		/* set all newly resized bytes zero */
	movl	__what_size_we_ask_dpmi_to_be, %ecx
	incl	%ecx						/* Size not limit */
	subl	%edi, %ecx			/* Adjust count for base */
	xorl	%eax, %eax
	testb	$0x80, __crt0_startup_flags
	jz	no_deadbeef
	movl	$0xdeadbeef, %eax		/* something really easy to spot */
no_deadbeef:
	shrl	$2, %ecx			/* div 4 Longwords not bytes */
	cld
	rep
	stosl
no_fill_sbrk_memory:

	movl	__what_size_we_ask_dpmi_to_be, %eax			/* store new size */
	incl	%eax
	movl	%eax, __what_size_dpmi_thinks_we_are

#if 0						/* No purpose */
	pushl	__dpmi_memory_handle
	pushl	__base_address
	pushl	__old_base_address
	movl	___sbrk_brk_hook, %eax
	call	%eax
	addl	$12, %esp
#endif

brk_nochange:					/* successful return */
	movl	__what_we_return_to_app_as_old_size, %eax
	jmp	brk_return

brk_error:					/* error return */
	movl	__what_we_return_to_app_as_old_size, %eax
	movl	%eax, __what_size_app_thinks_it_is
	movl	$-1, %eax

brk_return:
	movl	%eax, %ebx
	popl	%eax				/* restore interrupts */
	int	$0x31
	movl	%ebx, %eax

	popl	%ebx
	popl	%edi
	popl	%esi
	ret
