/*

	THIS IS THE FAT DS HACK that allows a DPMI client to access
	video memory with a near pointer. This version communicates with ___sbrk()
	in crt0.S to disable changes on the ds segment limit if the fat ds
	is in use. Stores requested limit value in _jhw_req_limit so the
	limit can be restored after finnished using the fat ds.
		This means that we can give up protection, and USE_FAT_DS()
	once. Or for the time that we need access to VRAM, we USE_FAT_DS(),
	do our blitting, then RESTORE_OLD_DS(). We are now allowed to use
	malloc() while using the fat ds.
		Junaid	19d/6m/95
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <gppconio.h>
#include <pc.h>
#include <dos.h>
#include <go32.h>
#include <dpmi.h>


#define MSIZE	100000	/*size to malloc while using fat ds hack*/

/*NO USER SERVICABLE CHANGES BELOW THIS LINE
----------------------------------------------------------------------------*/

#define FIXED	/*using my patched crt0.s that doesnt clobber the fat ds limit */



typedef unsigned char uchar;
typedef unsigned short ushort;
typedef unsigned uint;

#ifdef FIXED
extern uint _jhw_using_fat_ds;	/*We set it true when we used the fat ds, to prevent ___sbrk() from altering the current ds limit*/
extern uint _jhw_ds_req_limit;	/*last ds limit requested by ___sbrk()*/
#endif

uchar *graph_mem=NULL;
uint old_ds_sel=0;       /*old process's DS, and new unlimited 'fat DS'*/

static uint ds_sel, cs_sel, ss_sel, fs_sel, gs_sel;
static union REGS regs;

void tx(void)
	/*return to a cleared text mode screen*/
{
	    	regs.x.ax = 0x0003;	/*back to text mode*/
    		int86(0x10,&regs,&regs);
}

void gr(void)
	/*switch to GRaphics mode*/
{
		regs.x.ax = 0x0013;	/*mode 13 graphics*/
		int86(0x10,&regs,&regs);
}

void barf(char *msg)	/*Error message and exit*/
{	
	cprintf("\r\n%s\r\n", msg);
	exit(1);
}

static uint get_sel_limit(uint sel)
	/*get segment limit of the selector _sel */
{
	uint l;
    __asm__(
        "lsll 	%%eax,%%ebx\n\t"	/*get segment limit of %eax into %ebx*/
	"jz	1f\n\t"		/*lsl successful*/
	"movl	$0, %%ebx\n\t"	/*lsl unsuccessful => return bad limit of 0*/
    "1:\n\t"
		:"=b" (l) : "a" (sel) :"ax", "bx", "cc"
    );
    
    return l;
}

static void get_selectors(uint *code, uint *data, uint *stack,
        uint *frame, uint *gate)
	/*get current program's selectors */
{
    __asm__(
        "movw 	%%cs, %%ax\n\t"	/*put %cs selector into %ax */
	"movzwl	%%ax, %%ebx\n\t"	/*and zero extend %ax into %ebx*/
        "movw 	%%ds, %%ax\n\t"
	"movzwl	%%ax, %%ecx\n\t"
        "movw 	%%ss, %%ax\n\t"
	"movzwl	%%ax, %%edx\n\t"
        "movw 	%%fs, %%ax\n\t"
	"movzwl	%%ax, %%esi\n\t"
        "movw 	%%ds, %%ax\n\t"
	"movzwl	%%ax, %%eax\n\t"
		:"=b" (*code), "=c" (*data), "=d" (*stack), 
                 "=S" (*frame), "=a" (*gate)
		: /*no input*/ 
		:"ax", "bx", "cx", "dx", "si", "cc"
    );
}

static uint get_sel_base(uint sel)
{	/*figure out where selector is in physical memory*/
	regs.x.ax = 0x6;	/*get desrciptor linear base address*/
	regs.x.bx = sel;
	int86(0x31, &regs, &regs);
	if(regs.x.flags & 1) 
                cprintf("Bad base, DPMI fn 0x6, error=%08X\r\n", (int)regs.x.ax);
        return ((uint)(regs.x.cx)<<16) + (uint)(regs.x.dx);
}


static void set_sel_limit(uint sel, uint lim)
{
	regs.x.ax = 0x8;	/*set selector limit function*/
        regs.x.bx = sel;
	regs.x.cx = lim>>16;	/*high word of range*/
	regs.x.dx = lim;	/*low "*/
	int86(0x31, &regs, &regs);
        if(regs.x.flags & 1)
                cprintf("Couldnt set sel limit, DPMI fn 0x8, error=%08X\r\n", (int)regs.x.ax);
}

void USE_FAT_DS(void)
{
#ifdef FIXED
	_jhw_using_fat_ds=1;
#endif
	set_sel_limit(old_ds_sel, 0xffffffffUL);
}

void RESTORE_OLD_DS(void)
{
#ifdef FIXED
	set_sel_limit(old_ds_sel, _jhw_ds_req_limit);
	_jhw_using_fat_ds=0;
#endif
}

static void print_sel_bases(void)
{	/*print out all selectors bases*/
        get_selectors(&cs_sel, &ds_sel, &ss_sel, &fs_sel, &gs_sel);
        cprintf("Current linear base;CS=%08X ", get_sel_base(cs_sel));
	cprintf("DS=%08X ", get_sel_base(ds_sel));
	cprintf("SS=%08X ", get_sel_base(ss_sel));
	cprintf("FS=%08X ", get_sel_base(fs_sel));
	cprintf("GS=%08X\r\n", get_sel_base(gs_sel));
}


static void print_sels(void)
{	/*print out all selector values/limits*/
        get_selectors(&cs_sel, &ds_sel, &ss_sel, &fs_sel, &gs_sel);
        cprintf("Value/limit; CS=%08X/%08X DS=%08X/%08X SS=%08X/%08X\r\nFS=%08X/%08X GS=%08X/%08X\r\n",
                cs_sel, get_sel_limit(cs_sel), ds_sel, get_sel_limit(ds_sel),
                ss_sel, get_sel_limit(ss_sel), fs_sel, get_sel_limit(fs_sel),
                gs_sel, get_sel_limit(gs_sel) );
}

static void do_fat_ds_hack(void)
{       /*implement the fat ds hack to wrap around in virtual memory to the physical base of VRAM*/
        old_ds_sel=_go32_my_ds();
	print_sel_bases();
	print_sels();
}



int main(int argc, char **argv)
{
        uchar p1=200, p2;       /*pixels*/
        uint n;
	uchar *ptr;

if(_go32_info_block.run_mode == _GO32_RUN_MODE_DPMI) {
	cprintf("\r\nUSING DPMI. DPMI VERSION=%X\r\n", 
                (uint)(_go32_info_block.run_mode_info));
        do_fat_ds_hack();
} else {	/*must be using go32.exe (DJGPP V1.x) without dpmi*/
	graph_mem=(uchar *)0xd0000000L;
}

#ifdef FIXED
	cprintf("\r\n_jhw_ds_req_limit=%08X\r\n", _jhw_ds_req_limit);
#endif
	cprintf("YOU'LL SEE TOP LINE CHANGE. PRESS ANY KEY TO CONTINUE\r\n");
	getkey();	/*wait for keypress*/


        for(n=0; n<10; n+=2) {    /*lets write a few pixels to the screen*/
		tx();
		cprintf("\r\nBEFORE USE_FAT_DS():\r\n");
		print_sel_bases();
		print_sels();
                USE_FAT_DS();
		cprintf("AFTER USE_FAT_DS():\r\n");
		print_sel_bases();
		print_sels();
		if((ptr=(uchar *)malloc(MSIZE)) == NULL) barf("bad malloc");
		cprintf("AFTER MALLOC():\r\n");
		memset(ptr, 0x12, MSIZE);
		print_sel_bases();
		print_sels();
		graph_mem=(uchar *)(   0xb8000  -  ( get_sel_base(old_ds_sel)+0x10000 )  );
			/*Just before we access the screen, calculate the wrap around address (and recalculate after mallocs) :- 
			who knows what the 0x10000 fixup is for - size of the descriptor table??*/
        	graph_mem[n]=p1;
                p2=graph_mem[n];
                RESTORE_OLD_DS();
		cprintf("AFTER RESTORE_OLD_DS():\r\n");
		print_sel_bases();
		print_sels();
		getkey();	/*wait for keypress*/
		if(p2!=p1) barf("Bad fat ds mem access");        /*not pointing to ram*/
		if((ptr=(uchar *)malloc(MSIZE)) == NULL) barf("bad malloc");
		memset(ptr, 0x12, MSIZE);

                dosmemget(0xb8000+n, 1, &p2);        /*read pixel using a fn we know works*/
                if(p2 != p1) barf("Pixel didnt make it to screen");
        }

	barf("FINNISHED OK");
	return 0;
}











